import os

def getIncludeFiles( dirPath ):
    dir = os.listdir( dirPath )
    files = []
    
    for file in dir:
        filePath = os.path.join( dirPath, file )
        
        if os.path.isdir( filePath ):
            files = files + getIncludeFiles( filePath )
        elif filePath.endswith( '.inc' ):
            files.append( filePath )

    return files

def parseIncludeFile( filePath ):
    file = open( filePath, 'r' )
    fileLines = file.readlines()
    fileLines = [ line.strip() for line in fileLines ]
    fileLines.reverse()
    fileContent = ''
    
    functionName = ''
    functionHeader = ''
    functionDoc = ''
    functionParams = []
    getContent = False
    
    for index, line in enumerate( fileLines ):
        if line.startswith( 'stock' ) or line.startswith( 'native' ) or line.startswith( 'forward' ):
            if line.endswith( ')' ) == False and line.endswith( ';' ) == False:
                functionHeader = ''
                headerLine = ''
                i = 0
                
                while True:
                    headerLine += fileLines[ index - i ]
                    
                    if headerLine.endswith( ')' ) or headerLine.endswith( ';' ):
                        functionHeader += headerLine
                        break
                    else:
                        i += 1
            else:
                functionHeader = line
            
            functionHeader = functionHeader[ functionHeader.find( '(' ) + 1 : functionHeader.find( ')' ) ]
            
            functionName = line[ line.find( ' ' ) + 1 : line.find( '(' ) ]            
            if ':' in functionName:
                functionName = line[ line.find( ':' ) + 1 : line.find( '(' ) ]
            
            if index + 1 >= len( fileLines ) or fileLines[ index + 1 ].startswith( '*/' ) == False:
                fileContent += '        <KeyWord name="' + cleanXmlForInput(functionName) + '" func="yes">\n'
                fileContent += '            <Overload retVal=">" descr=""></Overload>\n'
                fileContent += '        </KeyWord>\n'
            else:
                getContent = True
            
        elif line.startswith( '/**' ) and getContent:
            
            fileContent += '        <KeyWord name="' + cleanXmlForInput(functionName) + '" func="yes">\n'
            fileContent += '            <Overload retVal=">" descr="\n'
            
            fileContent += 'Usage:\n' + cleanXmlForInput(getFunctionUsage( functionDoc ))
            fileContent += '\nNotes:' + cleanXmlForInput(getFunctionNotes( functionDoc ))
            
            if '@return' in functionDoc or '@noreturn' in functionDoc:
                fileContent += '\nReturn:\n' + cleanXmlForInput(getFunctionReturn( functionDoc ))
            
            if '@error' in functionDoc:
                fileContent += '\nOn error / Errors:\n' + cleanXmlForInput(getFunctionError( functionDoc ))
            
            fileContent += '">\n'
            
            functionParams = makeParamsList( functionHeader )
            for param in functionParams:
                fileContent += '                <Param name="' + cleanXmlForInput(param) + '"/>\n'
            
            fileContent += '            </Overload>\n'
            fileContent += '        </KeyWord>\n'
            
            functionName = ''
            functionHeader = ''
            functionDoc = ''
            functionParams = []
            getContent = False
        elif line.startswith( ' */' ):
            continue
        elif getContent:
            functionDoc += line[ 2: ] + '\n'
        
    file.close()
    
    return fileContent

def cleanXmlForInput( str ):
    return str.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;').replace('"', 'quot;').replace("'", '&apos;')
	
def makeParamsList( str ):
    list = str.split( ',' )
    list = [ line.replace( ' =', '=' ) for line in list ]
    list = [ line.replace( '= ', '=' ) for line in list ]
    list = [ line.replace( '=', ' = ' ) for line in list ]
    list = [ line.replace( '"', '\'' ) for line in list ]
    list = [ line.strip() for line in list ]
    return filter( None, list )

def getFunctionUsage( string ):
    list = string.split( '\n' )
    list = [ line.strip() for line in list ]
    list.reverse()
    ret = "\n".join( list )
    ret = ret.replace( '\t', '    ' )
    ret = ret.replace( '"', '\'' )
    ret = ret.replace( '<', '&lt;' )
    ret = ret.replace( '>', '&gt;' )
    ret = ret[ ret.find( '@param' ): ]
    
    if '@return' in ret:
        ret = ret[ :ret.find( '@return' ) ]
    
    elif '@noreturn' in ret:
        ret = ret[ :ret.find( '@noreturn' ) ]
    
    if '@error' in ret:
        ret = ret[ :ret.find( '@error' ) ]
    
    params = []
    list = ret.split( '@param ' )
    list = [ line.strip() for line in list ]
    list = filter( None, list )
    ret = ''
    
    for item in list:
        param = []
        param.append( item.split( ' ', 1 )[ 0 ] )
        
        if len( item.split( ' ', 1 ) ) == 1:
            param.append( '' )
        else:
            param.append( item.split( ' ', 1 )[ 1 ].strip() )
        params.append( param )
    
    if len( list ) == 0:
        return ''
    
    spacerCount = len( sorted( params, key = lambda param: len( param[ 0 ] ), reverse = True )[ 0 ][ 0 ] ) + 4
    
    for param in params:
        line = param[ 0 ]
        spacer = ' ' * ( spacerCount - len( line ) )
        line += spacer + param[ 1 ].replace( '\n', '\n' + ' ' * ( spacerCount + 4 ) )
        ret += '    ' + line + '\n'
    
    return ret.rstrip()

def getFunctionNotes( str ):
    list = str.split( '\n' )
    list = [ line.strip() for line in list ]
    list.reverse()
    ret = ''
    
    for line in list:
        if line.startswith( '@' ):
            break
        else:        
            ret += '    ' + line.strip() + '\n'
    
    ret = ret.replace( '"', '\'' )
    ret = ret.replace( '\t', '    ' )
    ret = ret.replace( '<', '&lt;' )
    ret = ret.replace( '>', '&gt;' )
    return ret.rstrip()

def getFunctionReturn( str ):
    list = str.split( '\n' )
    list = [ line.strip() for line in list ]
    ret = ''
    errorSection = '@error' in str
    returnSection = False
    
    for line in list:
        if errorSection and returnSection == False and line.startswith( '@error' ):
            returnSection = True
            errorSection = False
            continue
        
        if line.startswith( '@noreturn' ):
            return '    No return.'
        elif line.startswith( '@return' ):
            ret = '    ' + line.replace( '@return', '' ).strip() + '\n' + ret
            break
        
        ret = '    ' + line.strip() + '\n' + ret

    ret = ret.replace( '"', '\'' )
    return ret.rstrip()

def getFunctionError( str ):
    list = str.split( '\n' )
    list = [ line.strip() for line in list ]
    ret = ''
    
    for line in list:
        if line.startswith( '@error' ):
            ret = '    ' + line.replace( '@error', '' ).strip() + '\n' + ret
            break
        
        ret = '    ' + line.strip() + '\n' + ret

    ret = ret.replace( '"', '\'' )
    return ret.rstrip()

if __name__ == '__main__':
    includesPath = 'E:\\Servers\\local\\steamapps\\common\\Counter-Strike Global Offensive Beta - Dedicated Server\\csgo\\addons\\sourcemod\\scripting\\include'
    nppPath = 'E:\\'
    
    console.show()
    console.clear()
    console.write( '*** START ***\n' )
    console.write( 'includesPath = "' + includesPath + '"\n' )
    console.write( 'nppPath = "' + nppPath + '"\n' )
    
    files = getIncludeFiles( includesPath )
    
    xmlFileContent = ''
    xmlFileContent = '<?xml version="1.0" encoding="Windows-1252" ?>\n'
    xmlFileContent += '<NotepadPlus>\n'
    xmlFileContent += '    <AutoComplete language="sourcemod">\n'
    
    for file in files:
        xmlFileContent += parseIncludeFile( file )
    
    xmlFileContent += '    </AutoComplete>\n'
    xmlFileContent += '</NotepadPlus>\n'
    
    xmlFile = open( os.path.join( nppPath, 'sourcemod.xml' ), 'w' )
    xmlFile.writelines( xmlFileContent )
    xmlFile.close()
    
    console.write( '*** END ***\n' )